'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
* @Author: Zhi Wei (智伟)
* @Date:   2016-11-28
*/

const debug = (0, _debug2.default)('coupler:error');

function CPError(message, code, extra) {

  let lines = new Error().stack.split('\n');
  lines = lines.slice(3);
  lines.unshift(this.toString());

  this.code = code;
  this.message = message;
  this.stack = lines.join('\n');

  _lodash2.default.assign(this, extra);

  Error.call(this, message);
}

CPError.prototype = _lodash2.default.create(Error.prototype, {
  'name': 'CPError',
  'displayName': 'CPError',
  'constructor': CPError
});
Object.setPrototypeOf(CPError, Error);

CPError.prototype.toString = function () {
  const segs = [this.name, `: ${this.message}`];

  return segs.join('');
};

CPError.version = require('../package.json').version;

CPError.attachErrors = function (codes) {
  if (!codes) {
    return;
  }
  if (!this.attached) {
    this.attached = {};
  }

  _lodash2.default.forEach(codes, (opt, code) => {
    if (this.attached[code]) {
      return;
    }

    let name = _lodash2.default.upperFirst(_lodash2.default.camelCase(code));
    if (!_lodash2.default.endsWith(name, 'Error')) {
      name = name + 'Error';
    }

    opt = _lodash2.default.assign({}, opt);

    const msg = opt.message;
    const msgArgs = opt.message_arguments;
    const hasArgs = _lodash2.default.isArray(msgArgs);
    const tmp = hasArgs ? _lodash2.default.template(msg, { interpolate: /{{([\s\S]+?)}}/g }) : null;

    delete opt.message;
    delete opt.message_arguments;

    function Base(...args) {
      let message = msg;

      if (hasArgs && tmp) {
        const data = _lodash2.default.reduce(msgArgs, (res, arg, idx) => {
          res[arg] = args[idx];
          return res;
        }, {});
        message = tmp(data);
        args = args.slice(args.length - 1);
      }

      const extra = _lodash2.default.assign({}, opt, _lodash2.default.last(args));

      CPError.call(this, message || msg, code, extra);
    }

    Base.prototype = _lodash2.default.create(CPError.prototype, {
      'name': name,
      'displayName': name,
      'constructor': Base
    });
    Object.setPrototypeOf(Base, CPError);

    this[name] = Base;
    this.attached[code] = opt;
    debug(`error ${name} is attached`);
  });
};

exports.default = CPError;
module.exports = exports['default'];